<?php

namespace HS\IncomingMail\Processors;

use Nekman\LuhnAlgorithm\Contract\LuhnAlgorithmExceptionInterface;
use Nekman\LuhnAlgorithm\LuhnAlgorithmFactory;
use Nekman\LuhnAlgorithm\Number;

class Redactor
{
    /**
     * Attempt to match social security numbers with dashes.
     *
     * United States Social Security numbers are nine-digit numbers in the format AAA-GG-SSSS:
     * 1. The first three digits were historically (prior to mid-2011) assigned by geographical region, and are thus called the area number. The area number cannot be 000, 666, or between 900 and 999.
     * 2. Digits four and five are called the group number and range from 01 to 99.
     * 3. The last four digits are serial numbers from 0001 to 9999.
     * @var string
     */
    public string $socialDashRegex = '/\b(?<!\-)(?!666|000|9\d{2})\d{3}-(?!00)\d{2}-(?!0{4})\d{4}(?!\-)\b/';

    /**
     * Attempt to match social security numbers without dashes.
     * Basically 9 digits sitting alone in the note body. This could
     * have false positives, but it's better than nothing.
     * @var string
     */
    public string $socialNoDashRegex = '/\b\d{9}(?!\d|\-\d)\b/';

    /**
     * match 13-16 digit numbers that do not start with a + to remove phone numbers
     * @var string
     */
    public string $ccRegex = '/(?<!\+)\b(?:\d[ -]*){13,16}\b/';

    public function redact($string)
    {
        $redactions = $this->findRedactions($string);
        if ($redactions) {
            return $this->doRedaction($string, $redactions);
        }
        return $string;
    }

    public function luhnVerify($number)
    {
        try {
            $validator = Number::fromString($number);
            return LuhnAlgorithmFactory::create()->isValid($validator);
        } catch (\Exception|LuhnAlgorithmExceptionInterface $e) {
            return false;
        }
    }

    public function doRedaction($string, $redactions)
    {
        $redactions->each(function($redaction) use (&$string){
            if ($this->luhnVerify($redaction)) {
                $string = $this->replaceRedactions($redaction, $string);
            } else if (preg_match($this->socialDashRegex, $redaction)){
                $string = $this->replaceRedactions($redaction, $string);
            }else if (preg_match($this->socialNoDashRegex, $redaction)){
                $string = $this->replaceRedactions($redaction, $string);
            }
        });

        return $string;
    }

    protected function replaceRedactions($redaction, $string): string
    {
        return str_replace($redaction, str_repeat('█', mb_strlen($redaction)), $string);
    }

    public function creditCardRedactionOn(): bool
    {
        return (hs_setting('cHD_REDACTOR_CC') == 1);
    }

    public function socialRedactionOn(): bool
    {
        return (hs_setting('cHD_REDACTOR_SS') == 1);
    }

    public function socialNoDashRedactionOn(): bool
    {
        return (hs_setting('cHD_REDACTOR_SS_NO_DASH') == 1);
    }

    public function findRedactions($string)
    {
        $out = [];
        if ($this->socialRedactionOn()) {
            $out = array_merge($this->matchDashSSNs($string),$out);
        }
        if ($this->socialNoDashRedactionOn()) {
            $out = array_merge($this->matchNoDashSSNs($string),$out);
        }
        if ($this->creditCardRedactionOn()) {
            $out = array_merge($this->matchCreditCards($string),$out);
        }

        $items = collect($out)->collapse()->transform(function($item, $value){
            return trim($item);
        });

        return ($items->count() > 0) ? $items : false;
    }

    protected function matchCreditCards($string): array
    {
        preg_match_all($this->ccRegex, $string, $out);
        return $out;
    }

    protected function matchDashSSNs($string): array
    {
        preg_match_all($this->socialDashRegex, $string, $out);
        return $out;
    }

    protected function matchNoDashSSNs($string): array
    {
        preg_match_all($this->socialNoDashRegex, $string, $out);
        return $out;
    }
}
